# export blender model to soya

# HOW TO EXPORT A BLENDER MODEL :
# - run Blender and load your model
# - delete useless objects like camera, lights, armature that can crash this script
# - load the script and run it ! Your shape is edited in the Soya editor
# - you can configure the Soya's material, world, shape, ... path in the Soya editor

import os, os.path, tempfile

# WARNING : there are some hack on this script since Blender python module is very few documented...
#   (for example for vertices color)

import Blender

def pointbymatrix(p, m):
  return [p[0] * m[0][0] + p[1] * m[1][0] + p[2] * m[2][0] + m[3][0],
          p[0] * m[0][1] + p[1] * m[1][1] + p[2] * m[2][1] + m[3][1],
          p[0] * m[0][2] + p[1] * m[1][2] + p[2] * m[2][2] + m[3][2]]

def export_to_soya():
  objs      = Blender.Object.Get()
  code      = ""
  materials = []
  
  print objs
  
  for obj in objs:
    if(obj.data.block_type == "NMesh"): # export only NMesh
      nmesh = Blender.NMesh.GetRawFromObject(obj.name)
      
      matrix = [[obj.mat[0][0], obj.mat[0][1], obj.mat[0][2], obj.mat[0][3]],
                [obj.mat[1][0], obj.mat[1][1], obj.mat[1][2], obj.mat[1][3]],
                [obj.mat[2][0], obj.mat[2][1], obj.mat[2][2], obj.mat[2][3]],
                [obj.mat[3][0], obj.mat[3][1], obj.mat[3][2], obj.mat[3][3]]]
      
      for face in nmesh.faces:
        code = code + "f = soya.model.Face(root_world)\n"
        
        # face option
        if(face.smooth != 0): code = code + "f.smooth_lit = 1\n"
        #if(face.twoside != 0): code = code + "f.double_sided = 1\n"
        
        # material
        if(face.image != None):
          m_name = face.image.name[:face.image.name.find(".")]
          code += "f.material = mat_%s \n" % m_name
          for (already_name, already_image) in materials:
            if m_name == already_name: break
          else: materials.append((m_name, face.image.name))
          
        # vertices
        index = 0
        for vertex in face.v:
          # vertex coordinates
          co = pointbymatrix(vertex.co, matrix)
          code = code + "v = soya.model.Vertex(root_world, " + str(co[0]) + ", " + str(co[1]) + ", " + str(co[2]) + ")\n"
          
          # vertex color
          if(face.col != None and len(face.col) > 0):
            color = face.col[index]
            code = code + "v.color = (" + str(color.r / 255.0) + ", " + str(color.g / 255.0) + ", " + str(color.b / 255.0) + ", " + str(color.a / 255.0) + ")\n"
            
          # vertex texture coordinates
          if(len(face.uv) > 0):
            uv = face.uv[index]
            code = code + "v.tex_x = " + str(uv[0]) + "\nv.tex_y = " + str(1.0 - uv[1]) + "\n"
            
          code = code + "f.append(v)\n"
          index = index + 1
          
  full_code = """
# This file has been generated by the Soya Blender exporter

import Tkinter, sys, os, os.path, shutil, glob, soya, soya.model, soya.soya3d, soya.editor, soya.editor.main
soya.editor.main.App()

root_world = soya.soya3d.World()

# You may want to change these lines...
#soya.model.Image.PATH    = '/home/jiba/python/slune/images'
#soya.model.Material.PATH = '/home/jiba/python/slune/materials'
#soya.soya3d.World.PATH   = '/home/jiba/python/slune/worlds'
#soya.model.Shape.PATH    = '/home/jiba/python/slune/shapes'
"""
  
  for material, texture_filename in materials:
    full_code += """
try:
  mat_%(material)s = soya.model.Material.get('%(material)s')
except IOError: # New material
  mat_%(material)s = soya.model.Material('%(material)s')
  image = glob.glob(os.path.join(soya.model.Image.PATH, "%(material)s.*"))
  if image:
    mat_%(material)s.tex_filename = os.path.basename(image[0])
  soya.editor.edit(mat_%(material)s)
""" % locals()

  return full_code + "\n\n" + code + """
# This is to compensate because upper axis is Z in blender, Y in Soya
root_world.rotate_vertical(-90.0)

soya.editor.edit(root_world)

os.remove(sys.argv[0])

Tkinter.mainloop()
"""

filename = tempfile.mktemp(".py")
open(filename, "w").write(export_to_soya())

print "*****************", filename, "*****************"

os.system("python %s &" % filename)
